import 'package:basic_ui/src/models/basic_theme.dart';
import 'package:basic_ui/src/providers/theme_provider.dart';
import 'package:flutter/material.dart';
import 'package:flutter_hooks/flutter_hooks.dart';
import 'package:hooks_riverpod/hooks_riverpod.dart';

import 'basic_appbar.dart';

/// Creates a visual scaffold for page.
class BasicScaffold extends HookWidget {
  const BasicScaffold({
    Key? key,
    this.body,
    this.drawer,
    this.bottomNavigationBar,
    this.title,
    this.actions,
    this.floatingActionButton,
    this.appBarColor,
    this.backgroundColor,
    this.enableSearchBar,
    this.searchBarPlaceHolder,
    this.onClear,
    this.onSubmitted,
    this.disableAppBar = false,
    this.appBarElevation,
    this.tabs,
    this.tabBarViews,
  }) : super(key: key);

  /// The primary content of the scaffold.
  final Widget? body;

  /// A panel displayed to the side of the [body], often hidden on mobile devices.
  /// Swipes in from either left-to-right ([TextDirection.ltr])
  /// or right-to-left ([TextDirection.rtl]).
  final Widget? drawer;

  /// A bottom navigation bar to display at the bottom of the scaffold.
  final Widget? bottomNavigationBar;

  /// Display title to the app bar.
  final String? title;

  /// A list of Widgets to display in a row after the [title].
  final List<Widget>? actions;

  /// A button displayed floating above [body], in the bottom right corner.
  final Widget? floatingActionButton;

  /// [BasicAppBar] background color.
  final Color? appBarColor;

  /// [body] background color.
  final Color? backgroundColor;

  /// Set this to true to make app bar title display search bar.
  final bool? enableSearchBar;

  /// Search bar placeholder
  final String? searchBarPlaceHolder;

  /// The callback that is called when the clear button is tapped.
  final Function? onClear;

  /// The callback when search query submitted.
  final Function(String)? onSubmitted;

  /// Set this to true to disable app bar.
  final bool disableAppBar;

  // This property controls the size of the shadow below the app bar.
  // The value must be non-negative.
  // If this property is null, then [BasicTheme.appBarElevation] is used.
  // If that is also null, the default value is 0.
  final double? appBarElevation;

  /// The Widgets to display in tab bar,
  final List<Widget>? tabs;

  /// Creates a page view with one child per tab.
  ///
  /// The length of [tabBarViews] must be the same as the [tabs] length.
  final List<Widget>? tabBarViews;

  @override
  Widget build(BuildContext context) {
    final theme = useProvider(themeProvider).theme;

    if (tabs == null) {
      return _BuildScaffold(
          backgroundColor: backgroundColor,
          theme: theme,
          disableAppBar: disableAppBar,
          title: title,
          actions: actions,
          appBarColor: appBarColor,
          enableSearchBar: enableSearchBar,
          searchBarPlaceHolder: searchBarPlaceHolder,
          onClear: onClear,
          onSubmitted: onSubmitted,
          appBarElevation: appBarElevation,
          drawer: drawer,
          body: body,
          floatingActionButton: floatingActionButton,
          bottomNavigationBar: bottomNavigationBar);
    }

    return DefaultTabController(
      length: tabs!.length,
      child: Scaffold(
        backgroundColor: theme.backgroundColor,
        appBar: BasicAppBar(
          title: title ?? "",
          actions: actions,
          backgroundColor: appBarColor,
          enableSearchBar: enableSearchBar ?? false,
          searchBarPlaceHolder: searchBarPlaceHolder,
          onClear: onClear,
          onSubmitted: onSubmitted,
          tabs: tabs,
        ),
        drawer: drawer,
        body: TabBarView(children: tabBarViews!),
        floatingActionButton: floatingActionButton,
      ),
    );
  }
}

class _BuildScaffold extends StatelessWidget {
  const _BuildScaffold({
    Key? key,
    this.backgroundColor,
    required this.theme,
    this.disableAppBar = false,
    this.title,
    this.actions,
    this.appBarColor,
    this.enableSearchBar,
    this.searchBarPlaceHolder,
    this.onClear,
    this.onSubmitted,
    this.appBarElevation,
    this.drawer,
    this.body,
    this.floatingActionButton,
    this.bottomNavigationBar,
  }) : super(key: key);

  final Color? backgroundColor;
  final BasicTheme theme;
  final bool disableAppBar;
  final String? title;
  final List<Widget>? actions;
  final Color? appBarColor;
  final bool? enableSearchBar;
  final String? searchBarPlaceHolder;
  final Function? onClear;
  final Function(String)? onSubmitted;
  final double? appBarElevation;
  final Widget? drawer;
  final Widget? body;
  final Widget? floatingActionButton;
  final Widget? bottomNavigationBar;

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: backgroundColor ?? theme.backgroundColor,
      appBar: !disableAppBar
          ? BasicAppBar(
              title: title ?? "",
              actions: actions,
              backgroundColor: appBarColor,
              enableSearchBar: enableSearchBar ?? false,
              searchBarPlaceHolder: searchBarPlaceHolder,
              onClear: onClear,
              onSubmitted: onSubmitted,
              elevation: appBarElevation,
            )
          : null,
      drawer: drawer,
      body: body,
      floatingActionButton: floatingActionButton,
      bottomNavigationBar: bottomNavigationBar,
    );
  }
}
