import 'package:basic_ui/basic_ui.dart';
import 'package:basic_ui/src/providers/theme_provider.dart';
import 'package:flutter/material.dart';
import 'package:flutter_hooks/flutter_hooks.dart';
import 'package:hooks_riverpod/hooks_riverpod.dart';

import 'basic_text.dart';

/// Creates a [FormField] that contains a [TextField].
class BasicTextField extends HookWidget {
  const BasicTextField({
    Key? key,
    this.label,
    this.defaultValue,
    this.maxLength,
    this.keyboardType,
    this.onChanged,
    this.validator,
    this.isPassword = false,
    this.fillColor,
    this.textColor,
    this.labelColor,
    this.borderColor,
    this.focusedBorderColor,
    this.borderWidth = 1,
    this.borderRadius,
    this.contentPadding = const EdgeInsets.all(10),
  }) : super(key: key);

  /// Text to display as [BasicTextField] label.
  final String? label;

  /// Default value for [BasicTextField]
  final String? defaultValue;

  /// Set max length of [BasicTextField] value
  final int? maxLength;

  /// The type of information for which to optimize the text input control.
  ///
  /// On Android, behavior may vary across device and keyboard provider.
  ///
  /// This class stays as close to Enum interface as possible, and allows for additional
  /// flags for some input types. For example, numeric input can specify whether it supports
  /// decimal numbers and/or signed numbers.
  final TextInputType? keyboardType;

  /// The callback when [BasicTextField] value is changed.
  final Function(String)? onChanged;

  /// The callback when [FormField] is validated.
  final String? Function(String?)? validator;

  /// Set this to true to make [BasicTextField] hide it's value
  final bool isPassword;

  /// The base fill color of the decoration's container color.
  final Color? fillColor;

  /// Text color.
  final Color? textColor;

  /// Label color.
  final Color? labelColor;

  /// Border color.
  final Color? borderColor;

  /// Border color when [BasicTextField] is focused.
  final Color? focusedBorderColor;

  /// Border width.
  final double borderWidth;

  /// Border radius.
  final double? borderRadius;

  /// The padding for the input decoration's container.
  final EdgeInsetsGeometry contentPadding;

  @override
  Widget build(BuildContext context) {
    final theme = useProvider(themeProvider).theme;

    final _isPasswordShow = useState(false);
    final controller = useState(TextEditingController());

    useEffect(() {
      controller.value.text = defaultValue ?? "";
    }, []);

    return Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: [
        if (label != null)
          Padding(
            padding: const EdgeInsets.symmetric(vertical: 5),
            child: BasicText(
              label ?? "",
              variant: BasicTextVariant.label,
              color: labelColor,
            ),
          ),
        TextFormField(
          textInputAction: TextInputAction.go,
          controller: controller.value,
          onChanged: onChanged,
          enableSuggestions: false,
          autocorrect: false,
          maxLength: maxLength,
          keyboardType: keyboardType,
          validator: validator,
          obscureText: isPassword && !_isPasswordShow.value,
          decoration: InputDecoration(
            isDense: true,
            filled: fillColor != null,
            fillColor: fillColor,
            contentPadding: contentPadding,
            focusedBorder: OutlineInputBorder(
              borderSide: BorderSide(
                color: borderWidth != 0
                    ? focusedBorderColor ?? theme.primaryColor
                    : Colors.transparent,
                width: borderWidth != 0 ? borderWidth + .5 : 0,
              ),
              borderRadius: BorderRadius.all(
                Radius.circular(borderRadius ?? 3),
              ),
            ),
            enabledBorder: OutlineInputBorder(
              borderSide: BorderSide(
                color: borderWidth != 0
                    ? focusedBorderColor ?? borderColor ?? theme.fontColor
                    : Colors.transparent,
                width: borderWidth,
              ),
              borderRadius: BorderRadius.all(
                Radius.circular(borderRadius ?? 3),
              ),
            ),
            errorBorder: OutlineInputBorder(
              borderSide: BorderSide(
                color: theme.dangerColor,
                width: borderWidth,
              ),
              borderRadius: BorderRadius.all(
                Radius.circular(borderRadius ?? 3),
              ),
            ),
            focusedErrorBorder: OutlineInputBorder(
              borderSide: BorderSide(
                color: theme.dangerColor,
                width: borderWidth,
              ),
              borderRadius: BorderRadius.all(
                Radius.circular(borderRadius ?? 3),
              ),
            ),
            errorStyle: TextStyle(color: theme.dangerColor),
            suffixIcon: BasicTap(
              onTap: () {
                if (!isPassword) {
                  controller.value.clear();
                } else {
                  _isPasswordShow.value = !_isPasswordShow.value;
                }
              },
              child: Icon(
                !isPassword
                    ? Icons.close
                    : _isPasswordShow.value
                        ? Icons.visibility_off
                        : Icons.visibility,
                color: textColor ?? theme.fontColor,
              ),
            ),
            suffixIconConstraints: BoxConstraints(
              minWidth: contentPadding.horizontal + 29,
              minHeight: contentPadding.vertical + 29,
            ),
          ),
          style: TextStyle(
            fontSize: 14,
            color: textColor ?? theme.fontColor,
          ),
        ),
      ],
    );
  }
}
